import matplotlib.pyplot as mp
import numpy as np

import projection as proj
import  fov

#import scenery

def main():
    wantPlot = True
    zoom = False

    ra_deg = 98.15766666666666
    dec_deg = 21.594944444444444
    scRoll_deg = 157.6053108

    fovRoll_deg = fov.getFovAngleFromSpacecraftRoll(scRoll_deg)
    k = fov.KeplerFov(ra_deg, dec_deg, fovRoll_deg)

    #Position of example star, WASP-67
    ra_deg, dec_deg = 103.67845833333334, 24.245555555555555


    #Get coordinates of channel corners.
    raDec = k.getCoordsOfChannelCorners()

    #Is a given ra/dec on silicon?
    try:
        ch = k.pickAChannel(ra_deg, dec_deg)
        print "Star is on channel %i Mod-out %s" %(ch, fov.modOutFromChannel(ch))
    except ValueError:
        print "Star is not on silicon"
        print "Caution: Stars within 5 pix of channel edges are not found by this algorithm"


    ch, col, row = k.getChannelColRow(ra_deg, dec_deg)
    print "Star is on channel %i Col %.1f Row %.1f" %(ch, col, row)


    if wantPlot:
        mp.clf()
        #ph is a projection class, in this case HammerAitoff. Pass this
        #class to all FOV plotting commands so it knows how to do the
        #plotting. ph also implements matplotlib's plot, scatter and text
        #commands
        #ph = proj.HammerAitoff(0,0)
        ph = proj.Cylindrical()
        ph.plotGrid()

        k.plotPointing(ph)

        #scenery.plotEcliptic(ph)
        plotWithLabel(ph, ra_deg, dec_deg, "HD 50554")

        if zoom:
            mp.axis([.88, 1.36, -0.60, -0.20])
            k.plotChIds(ph, modout=True)
        mp.show()





def plotWithLabel(maptype, ra, dec, label):
    x,y = maptype.skyToPix(ra, dec)
    mp.plot(x, y, 'ro')
    mp.text(x, y, label)


if __name__ == "__main__":
    main()



"""
Created on Wed Jan 29 10:40:14 2014

@author: tom_barclay


The code will accept a file containing a minimum of two columns
RA Dec.
where RA and Dec are in decimal degrees

"""
from __future__ import division, print_function
import sys
try:
    import numpy as np
except ImportError:
    print('You need numpy installed')
    sys.exit(1)

try:
    import matplotlib.pyplot as plt
    got_mpl = True
except ImportError:
    print('You need matplotlib installed to get a plot')
    got_mpl = False

import projection as proj
import  fov

__version__ = '0.0.1'

params = {#'backend': 'png',
                        'axes.linewidth': 1.5,
                        'axes.labelsize': 24,
                        'axes.font': 'sans-serif',
                        #'axes.fontweight' : 'bold',
                        'text.fontsize': 22,
                        'legend.fontsize': 14,
                        'xtick.labelsize': 16,
                        'ytick.labelsize': 16,
                        'text.usetex': False,
                        #'font.family': 'Palatino'
                        }
plt.rcParams.update(params)

#try:
#    plt.rcParams.update({'font.family': 'Palatino'})
#except:
#    pass


def angSepVincenty(ra1,dec1,ra2,dec2):
    """
    Vincenty formula for distances on a sphere
    """
    ra1_rad = np.radians(ra1)
    dec1_rad = np.radians(dec1)
    ra2_rad = np.radians(ra2)
    dec2_rad = np.radians(dec2)
    #diffpos = np.arccos(
    #    np.sin(dec1_rad)*np.sin(dec2_rad) +
    #    np.cos(dec1_rad)*np.cos(dec2_rad)*np.cos(np.abs(ra1_rad - ra2_rad)))

    sin_dec1, cos_dec1 = np.sin(dec1_rad), np.cos(dec1_rad)
    sin_dec2, cos_dec2 = np.sin(dec2_rad), np.cos(dec2_rad)
    delta_ra = ra2_rad - ra1_rad
    cos_delta_ra, sin_delta_ra = np.cos(delta_ra), np.sin(delta_ra)

    diffpos = np.arctan2(np.sqrt((cos_dec2 * sin_delta_ra) ** 2 +
                       (cos_dec1 * sin_dec2 -
                        sin_dec1 * cos_dec2 * cos_delta_ra) ** 2),
                  sin_dec1 * sin_dec2 + cos_dec1 * cos_dec2 * cos_delta_ra)

    return np.degrees(diffpos)


def parse_file(infile):
    try:
        a,b, mag = np.atleast_2d(np.genfromtxt(infile, usecols=[0,1,2],delimiter=',')).T
    except IOError:
        print('There seems to be a problem with the input file, the format should be: RA_degrees (J2000), \
            Dec_degrees (J2000), Magnitude. There should be no header, columns should be seperated by a comma')
        sys.exit(1)
    return a,b, mag

def onSiliconCheck(ra_deg,dec_deg,FovObj):
    try:
        ch = FovObj.pickAChannel(ra_deg, dec_deg)
        ch, col, row = FovObj.getChannelColRow(ra_deg, dec_deg)
        #exclude modules 3 and 7
        if ch in [5,6,7,8,17,18,19,20]:
            return False
        #return (ch,col,row)
        return True
    except ValueError:
        return False

def nearSiliconCheck(ra_deg,dec_deg,FovObj,max_sep=8.2):
    dist = angSepVincenty(FovObj.ra0_deg,FovObj.dec0_deg,ra_deg,dec_deg)
    if dist <= max_sep:
        return True
    else:
        return False


def getRaDecRollFromFieldnum(fieldnum):
    if fieldnum not in [0,1,2,3,4]:
        raise ValueError('Only Fields 0-4 are defined in this version of the code')
    elif fieldnum == 0:
        #ra_deg = 98.15766666666666
        #dec_deg = 21.594944444444444
        #scRoll_deg = 177.535
        ra_deg = 98.2964079
        dec_deg = 21.5878901
        scRoll_deg = 177.4810830
    elif fieldnum == 1:
        ra_deg = 173.939610
        dec_deg = 1.4172989
        scRoll_deg = 157.641206
    elif fieldnum == 2:
        ra_deg = 246.1264
        dec_deg = -22.4473
        scRoll_deg = 171.2284
    elif fieldnum == 3:
        ra_deg = 336.665346414
        dec_deg = -11.096663792
        scRoll_deg = -153.494818181
    elif fieldnum == 4:
        print('Warning, Field 4 position will change')
        ra_deg = 56.496
        dec_deg = 18.130472222222224
        scRoll_deg = 177.4810830
    else:
        raise NotImplementedError

    return (ra_deg, dec_deg, scRoll_deg)


# def run_test():
#     infile = 'KeplerCampaignMDwarfsTargetList.txt'
#     fieldnum = 0

#     ra_sources_deg, dec_sources_deg = parse_file(infile)
#     ra_deg, dec_deg, scRoll_deg = getRaDecRollFromFieldnum(fieldnum)
#     fovRoll_deg = fov.getFovAngleFromSpacecraftRoll(scRoll_deg)

#     k = fov.KeplerFov(ra_deg, dec_deg, fovRoll_deg)

#     raDec = k.getCoordsOfChannelCorners()


#     onSilicon = map(onSiliconCheck,
#         ra_sources_deg,dec_sources_deg,np.repeat(k,len(ra_sources_deg)))

#     nearSilicon = map(nearSiliconCheck,
#         ra_sources_deg,dec_sources_deg,np.repeat(k,len(ra_sources_deg)))

#     onSilicon = np.array(onSilicon,dtype=bool)
#     nearSilicon = np.array(nearSilicon, dtype=bool)

#     if got_mpl:
#         ph = proj.Cylindrical()
#         k.plotPointing(ph,showOuts=False)
#         targets = ph.skyToPix(ra_sources_deg, dec_sources_deg)
#         fig = plt.gcf()
#         ax = fig.gca()
#         ax = fig.add_subplot(111)
#         ax.scatter(*targets,s=0.5)
#         ax.scatter(targets[0][nearSilicon],
#             targets[1][nearSilicon],color='b')
#         ax.scatter(targets[0][onSilicon],
#             targets[1][onSilicon],color='r')
#         ax.set_xlabel('R.A. [radians]',fontsize=16)
#         ax.set_ylabel('Declination [radians]',fontsize=16)




#         fig.show()


def K2onSilicon(infile,fieldnum):
    ra_sources_deg, dec_sources_deg, mag = parse_file(infile)

    if np.shape(ra_sources_deg)[0] > 500:
        print('There are {} sources in your target list, this could take some time'.format(np.shape(ra_sources_deg)[0]))

    ra_deg, dec_deg, scRoll_deg = getRaDecRollFromFieldnum(fieldnum)

    ## convert from SC roll to FOV coordinates
    ## do not use the fovRoll coords anywhere else
    ## they are internal to this script only
    fovRoll_deg = fov.getFovAngleFromSpacecraftRoll(scRoll_deg)

    ## initialize class
    k = fov.KeplerFov(ra_deg, dec_deg, fovRoll_deg)

    raDec = k.getCoordsOfChannelCorners()


    onSilicon = map(onSiliconCheck,
        ra_sources_deg,dec_sources_deg,np.repeat(k,len(ra_sources_deg)))

    nearSilicon = map(nearSiliconCheck,
        ra_sources_deg,dec_sources_deg,np.repeat(k,len(ra_sources_deg)))

    onSilicon = np.array(onSilicon,dtype=bool)
    nearSilicon = np.array(nearSilicon, dtype=bool)

    if got_mpl:
        almost_black = '#262626'
        light_grey = np.array([float(248)/float(255)]*3)
        ph = proj.Cylindrical()
        k.plotPointing(ph,showOuts=False,plot_degrees=True)
        targets = ph.skyToPix(ra_sources_deg, dec_sources_deg)
        targets = np.array(targets ) * 180 / np.pi
        fig = plt.gcf()
        ax = fig.gca()
        ax = fig.add_subplot(111)
        ax.scatter(*targets,s=2,label='not on silicon')
        ax.scatter(targets[0][nearSilicon],
            targets[1][nearSilicon],color='#fc8d62',s=8,label='near silicon')
        ax.scatter(targets[0][onSilicon],
            targets[1][onSilicon],color='#66c2a5',s=8,label='on silicon')
        ax.set_xlabel('R.A. [degrees]',fontsize=16)
        ax.set_ylabel('Declination [degrees]',fontsize=16)
        ax.invert_xaxis()
        ax.minorticks_on()
        legend = ax.legend(loc=0,
            frameon=True, scatterpoints=1)
        rect = legend.get_frame()
        rect.set_alpha(0.3)
        rect.set_facecolor(light_grey)
        rect.set_linewidth(0.0)
        texts = legend.texts
        for t in texts:
            t.set_color(almost_black)
        fig.savefig('targets_fov.png',dpi=300)
        plt.close('all')

    siliconFlag = np.zeros_like(ra_sources_deg)
    siliconFlag = np.where(nearSilicon,1,siliconFlag)
    siliconFlag = np.where(onSilicon,2,siliconFlag)

    outarr = np.array([ra_sources_deg, dec_sources_deg, mag, siliconFlag])
    np.savetxt('targets_siliconFlag.csv', outarr.T, delimiter=', ',
        fmt=['%10.10f','%10.10f','%10.2f','%i'])

    if got_mpl:
        print('I made two files: targets_siliconFlag.csv and targets_fov.png')
    else:
        print('I made one file: targets_siliconFlag.csv')





if __name__ == '__main__':
    if len(sys.argv) == 1:
        print('')
        print('python K2onSilicon.py filename fieldnum')
        print('')
        sys.exit(2)
    elif len(sys.argv) != 3:
        print('use the command')
        print('python K2onSilicon.py filename fieldnum')
        sys.exit(2)

    fieldnum = int(sys.argv[2])
    infile = str(sys.argv[1])

    K2OnSilicon(infile, fieldnum)











#!/usr/bin/env python


try:
    from setuptools import setup
    setup
except ImportError:
    from distutils.core import setup
    setup

setup(name='K2fov',
      version='1.2.4',
      description='Find which targets are in the field of view of K2',
      author='Tom Barclay',
      author_email='tom@tombarclay.com',
      url='https://github.com/mrtommyb/K2fov',
      packages=['K2fov'],
      install_requires=["numpy"],)





##find out if targets are on silicon

from __future__ import division
import sys
sys.path.append('/Users/tom/gitcode/K2_fov/K2_fov/')
import numpy as np
import matplotlib.pyplot as plt
import K2onSilicon

import csv

if __name__ == '__main__':
    fieldnum = 0

    infile = 'GO00.csv'

    r = csv.reader(open('GO00.csv','rU'))

    t  = [x for x in r]
    t1  = np.array(t).T

    ra_sources_deg = t1[3,1:].astype('float')
    dec_sources_deg = t1[4,1:].astype('float')

    ra_deg, dec_deg, scRoll_deg = K2onSilicon.getRaDecRollFromFieldnum(
        fieldnum)

    fovRoll_deg = K2onSilicon.fov.getFovAngleFromSpacecraftRoll(scRoll_deg)
    k = K2onSilicon.fov.KeplerFov(ra_deg, dec_deg, fovRoll_deg)

    raDec = k.getCoordsOfChannelCorners()

    onSilicon = map(K2onSilicon.onSiliconCheck,
        ra_sources_deg,dec_sources_deg,np.repeat(k,len(ra_sources_deg)))

    nearSilicon = map(K2onSilicon.nearSiliconCheck,
        ra_sources_deg,dec_sources_deg,np.repeat(k,len(ra_sources_deg)))

    onSilicon = np.array(onSilicon,dtype=bool)
    nearSilicon = np.array(nearSilicon, dtype=bool)

    ##savetable

    ### doesnt' work properly!!!
    SiliconFlag = np.zeros_like(onSilicon,dtype=bool)
    SiliconFlag = np.where(nearSilicon,1,SiliconFlag)
    SiliconFlag = np.where(onSilicon,2,SiliconFlag)
    SiliconFlag = np.insert(SiliconFlag.astype(str),0,'SiliconFlag')
    outfile = 'GO00_Silicon.csv'
    newarr = np.zeros([np.shape(t1)[0]+1,np.shape(t1)[1]],dtype=str)
    newarr[0:np.shape(t1)[0],0:np.shape(t1)[1]] = t1
    newarr[-1] = SiliconFlag

    np.savetxt('GO00_Silicon.csv', newarr.T)




    do_plot = True
    do_degrees = True
    if do_plot:
        ph = K2onSilicon.proj.Cylindrical()
        targets = ph.skyToPix(ra_sources_deg, dec_sources_deg)
        if do_degrees:
            k.plotPointing(ph,showOuts=False,plot_degrees=True)
            fig = plt.gcf()
            ax = fig.gca()
            ax = fig.add_subplot(111)
            ax.scatter(np.degrees(targets[0]),
                np.degrees(targets[1]),s=0.5)
            ax.scatter(np.degrees(targets[0][nearSilicon]),
                np.degrees(targets[1][nearSilicon]),color='b',s=1)
            ax.scatter(np.degrees(targets[0][onSilicon]),
                np.degrees(targets[1][onSilicon]),color='r',s=1)

            ax.set_xlabel('R.A. [degrees]',fontsize=16)
            ax.set_ylabel('Declination [degrees]',fontsize=16)

        else:
            k.plotPointing(ph,showOuts=False)
            fig = plt.gcf()
            ax = fig.gca()
            ax = fig.add_subplot(111)
            ax.scatter(targets[0],
                targets[1],s=0.5)
            ax.scatter(targets[0][nearSilicon],
                targets[1][nearSilicon],color='b',s=1)
            ax.scatter(targets[0][onSilicon],
                targets[1][onSilicon],color='r',s=1)

            ax.set_xlabel('R.A. [radians]',fontsize=16)
            ax.set_ylabel('Declination [radians]',fontsize=16)





try:
    import matplotlib.pyplot as mp
except ImportError:
    pass
import numpy as np
import rotate

__version__ = "$Id: projection.py 36 2014-01-23 22:19:15Z fergalm $"
__URL__ = "$URL: http://svn.code.sf.net/p/keplertwowheel/code/py/projection.py $"


class Projection():
    """Base Projection class. Used for mapping ra and dec into
    Euclidean space based on a given projection.

    The primary reference for projections is Calabretta and Greinstein
    (2002), A&A 395, 1077

    The base class implements the Plate Carree projection (\S 5.2.3)
    which just maps ra dec to xy -- i.e what you would blindly do
    if you knew no different. If distortion is not a concern
    this is an acceptable approach

    """
    def __init__(self):
        pass

    def skyToPix(self, ra_deg, dec_deg):
        return ra_deg, dec_deg

    def pixToSky(self, x, y):
        return x, y

    def eulerRotate(self, ra_deg, dec_deg):
        ra_deg, dec_deg = self.parseInputs(ra_deg, dec_deg)

        #Transform ra dec into angle away from tangent point
        #using the rotation matrix
        theta_rad= np.empty( (len(ra_deg),) )
        phi_rad = theta_rad * 0
        R = self.Rmatrix
        for i in range(len(ra_deg)):
            #Convert the ra/dec to a vector, then rotate so
            #that the tangent point is at [1,0,0]. Then pull out
            #the angle relative to the x-axis, and the angle
            #around the y-z plane.
            #@TODO: Can I make this faster with dot products?
            vec =rotate.vecFromRaDec(ra_deg[i], dec_deg[i])
            aVec = np.dot(R, vec)

            #aVec = (sint, cost*cosp, cost*sinp)
            sint = aVec[0]
            cost = np.hypot(aVec[1], aVec[2])
            theta = np.arctan2(sint, cost)

            cost = np.cos(theta)
            cosp = aVec[1] / cost
            sinp = aVec[2] / cost
            phi = np.arctan2(sinp, cosp)

            if phi < 0:
                phi += 2*np.pi
            if phi > 2*np.pi:
                phi -= 2*np.pi


            #Just to be explicit
            theta_rad[i] = theta
            phi_rad[i] = phi
        return theta_rad, phi_rad



    def parseInputs(self, ra_deg, dec_deg):
        try:
            len(ra_deg)
        except TypeError:
            ra_deg = np.array([ra_deg])

        try:
            len(dec_deg)
        except TypeError:
            dec_deg = np.array([dec_deg])

        #If ra/dec aren't arrays, make them arrays
        if not isinstance(ra_deg, np.ndarray):
            ra_deg = np.array(ra_deg)

        if not isinstance(dec_deg, np.ndarray):
            dec_deg = np.array(dec_deg)


        if np.logical_xor(len(ra_deg) == 1, len(dec_deg) == 1):
            if len(ra_deg) == 1:
                ra_deg = dec_deg *0 + ra_deg[0]
            else:
                dec_deg = ra_deg * 0 + dec_deg[0]

        if len(ra_deg) != len(dec_deg):
            raise ValueError("Input ra and dec arrays must be same length")

        return ra_deg, dec_deg


    def plot(self, ra_deg, dec_deg, *args, **kwargs):
        x,y = self.skyToPix(ra_deg, dec_deg)
        try:
            plot_degrees = kwargs.pop('plot_degrees')
        except KeyError:
            plot_degrees=False
        if plot_degrees:
            x,y = np.degrees(x), np.degrees(y)
        self._plot(x, y, *args, **kwargs)

    def scatter(self,  ra_deg, dec_deg, *args, **kwargs):
        x,y = self.skyToPix(ra_deg, dec_deg)
        mp.scatter(x,y, *args, **kwargs)


    def text(self, ra_deg, dec_deg, s, *args, **kwargs):
        x,y = self.skyToPix(ra_deg, dec_deg)
        mp.text(x, y, s, *args, **kwargs)

    def plotGrid(self, lineWidth=1, stepInDegrees=15, colour="#777777", \
        raRange=[0,360], decRange=[-90, 90]):

        ra0, ra1 = raRange
        dec0, dec1 = decRange
        step=stepInDegrees

        c = colour
        ra_deg = np.arange(ra0-1*step, ra1+1.5*step, 1, dtype=np.float)
        for dec in np.arange(dec0, dec1+ 1*step, step):
            self.plotLine(ra_deg, dec, '-', color=c, linewidth=lineWidth)

        dec = np.arange(dec0-step, dec1+1.5*step, 1, dtype=float)
        for ra in np.arange(ra0, ra1+step, step):
            self.plotLine(ra, dec, '-', color=c, linewidth=lineWidth)

        ##Useful for debugging
        #self.plotLine(0, dec,'r-', linewidth=lineWidth)
        #self.plotLine(180, dec,'c-', linewidth=lineWidth)


    def plotLine(self, ra_deg, dec_deg, *args, **kwargs):
        ra_deg, dec_deg = self.parseInputs(ra_deg, dec_deg)
        x,y = self.skyToPix(ra_deg, dec_deg)

        diffX = np.abs(np.diff(x))
        idx1 = diffX > 3*np.mean(diffX)
        idx1[idx1 + 1] = True

        diffY = np.abs(np.diff(y))
        idx2 = diffY > 3*np.mean(diffY)

        j = 0
        i0 = 0
        if len(idx2) > 0:
            idx2[-1] = True

        idx = np.where(np.logical_or(idx1, idx2))[0]
        for j in range(len(idx)):
            i1 = idx[j]
            self._plot(x[i0:i1], y[i0:i1], *args, **kwargs)
            i0 = i1+1



    def _plot(self, x, y, *args,  **kwargs):
        mp.plot(x,y, *args, **kwargs)



class PlateCaree(Projection):
    """Synonym for the base class"""
    pass


class HammerAitoff(Projection):
    def __init__(self, ra0_deg, dec0_deg):
        Projection.__init__(self)
        self.ra0_deg = ra0_deg
        self.dec0_deg = dec0_deg

        self.ra0_deg = ra0_deg
        self.dec0_deg = dec0_deg

        #This projection assumes ra ranges from -180 to +180
        #if self.ra0_deg > 180:
            #self.ra0_deg -= 360

        #Construct rotation matrix used to convert ra/dec into
        #angle relative to tangent point
        Rdec = rotate.declinationRotationMatrix(-self.dec0_deg)
        Rra = rotate.rightAscensionRotationMatrix(-self.ra0_deg)
        self.Rmatrix = np.dot(Rra, Rdec)



    def skyToPix(self, ra_deg, dec_deg):
        sin = np.sin
        cos = np.cos

        #Parse inputs and allocate space for outputs
        ra_deg, dec_deg = self.parseInputs(ra_deg, dec_deg)
        long_deg = ra_deg * 0
        lat_deg = long_deg * 0

        #Get longitude and latitude relative to defined origin.
        for i in range(len(ra_deg)):
            vec = rotate.vecFromRaDec(ra_deg[i], dec_deg[i])
            aVec = np.dot( self.Rmatrix, vec)
            long_deg[i], lat_deg[i] = rotate.raDecFromVec(aVec)

        long_deg = np.fmod(long_deg + 180, 360.)
        long_rad = np.radians(long_deg) - np.pi #[-pi,pi]
        lat_rad = np.radians(lat_deg)

        #long_rad = np.fmod(long_rad+ np.pi, 2*np.pi)

        gamma = 1 + cos(lat_rad)* cos(long_rad/2.)
        gamma = np.sqrt(2/gamma)
        x = -2*gamma*cos(lat_rad)*sin(long_rad/2)
        y = gamma*sin(lat_rad)

        return x, y


    def pixToSky(self, x, y):
        raise NotImplementedError("pixToSky not defined!")




class Gnomic(Projection):
    def __init__(self, ra0_deg, dec0_deg):
        self.ra0_deg = ra0_deg
        self.dec0_deg = dec0_deg

        #Construct rotation matrix used to convert ra/dec into
        #angle relative to tangent point
        Rdec = rotate.declinationRotationMatrix(-self.dec0_deg)
        Rra = rotate.rightAscensionRotationMatrix(-self.ra0_deg)
        self.Rmatrix = np.dot(Rdec, Rra)


        #Check I created the matrix correctly.
        origin = rotate.vecFromRaDec(self.ra0_deg, self.dec0_deg)
        origin = np.dot(self.Rmatrix, origin)
        assert( np.fabs(origin[0] -1 ) < 1e-9)
        assert( np.fabs(origin[1]) < 1e-9)
        assert( np.fabs(origin[2]) < 1e-9)


    def skyToPix(self, ra_deg, dec_deg):
        ra_deg, dec_deg = self.parseInputs(ra_deg, dec_deg)

        #Transform ra dec into angle away from tangent point
        #using the rotation matrix
        theta_rad= np.empty( (len(ra_deg),) )
        phi_rad = theta_rad * 0
        R = self.Rmatrix
        for i in range(len(ra_deg)):
            #Convert the ra/dec to a vector, then rotate so
            #that the tangent point is at [1,0,0]. Then pull out
            #the angle relative to the x-axis, and the angle
            #around the y-z plane.
            #@TODO: Can I make this faster with dot products?
            vec =rotate.vecFromRaDec(ra_deg[i], dec_deg[i])
            aVec = np.dot(R, vec)

            #aVec = (sint, cost*cosp, cost*sinp)
            sint = aVec[0]
            cost = np.hypot(aVec[1], aVec[2])
            theta = np.arctan2(sint, cost)

            cost = np.cos(theta)
            cosp = aVec[1] / cost
            sinp = aVec[2] / cost
            phi = np.arctan2(sinp, cosp)

            if phi < 0:
                phi += 2*np.pi
            if phi > 2*np.pi:
                phi -= 2*np.pi


            #Just to be explicit
            theta_rad[i] = theta
            phi_rad[i] = phi


        #Project onto tangent plane
        #theta_rad = np.pi/2. - theta_rad
        r = 1/(np.tan(theta_rad) + 1e-10) #Prevent division by zero
        x = r * np.cos(phi_rad)
        y = r * np.sin(phi_rad)

        #print x, y
        return x, y


    def pixToSky(self, x, y):
        x, y = self.parseInputs(x, y)

        R = self.Rmatrix
        invR = np.matrix(R.transpose())
        ra_deg = np.empty( (len(x),))
        dec_deg = np.empty( (len(x),))

        for i in range(len(x)):
            phi_rad = np.arctan2(y,x)
            r = np.hypot(x,y)
            theta_rad = np.arctan(r)

            aVec = np.zeros((3,))
            aVec[0] = np.cos(theta_rad)
            aVec[1] = np.sin(theta_rad)*np.cos(phi_rad)
            aVec[2] = np.sin(theta_rad)*np.sin(phi_rad)


            vec = np.dot(invR, aVec)
            vec = np.array(vec)[0]    #Convert to 1d array
            ra_deg[i], dec_deg[i] = rotate.raDecFromVec(vec)
        return ra_deg, dec_deg



class Cylindrical(Projection):
    """Stunted cyclindical projection that assumes
    projection point is always at sky point 0,0
    """
    def __init__(self):
        self.ra0_deg = 0
        self.dec0_deg = 0


    def skyToPix(self, ra_deg, dec_deg):
        x = np.radians(ra_deg)
        y = np.sin( np.radians(dec_deg))
        return x, y

    def pixToSky(self, x, y):
        ra = np.degrees(x)
        dec = np.degrees(np.arcsin(y))
        return ra, dec


def main():
    mp.clf()
    p = HammerAitoff(45,23)
    #p = Projection()

    #import pdb; pdb.set_trace()
    #print p.skyToPix(30,0)
    #print p.skyToPix(30,30)
    #print p.skyToPix(90, 30)
    #print p.skyToPix(181,30)

    #print p.skyToPix(270,0)

    #ra = np.arange(-180, 180, 15)
    #p.plot(ra, 0, 'r-')
    #p.plot(ra, 30, 'g-')
    #p.plot(ra, 45, 'b-')
    #p.plot(ra, 75, 'c-')
    #p.plot(ra, 90, 'k-')

    #dec = np.arange(-90, 91, 5)
    #p.plot(0, dec, 'r-')
    #p.plot(45, dec, 'g-')
    #p.plot(90, dec, 'b-')
    #p.plot(135, dec, 'c-')
    #p.plot(180, dec, 'k-')


    #p.plot(225, dec, 'g-')
    #p.plot(270, dec, 'b-')
    #p.plot(315, dec, 'c-')
    #p.plot(350, dec, 'k-')

    #conv = {3: tools.toFloat}
    #filename = "../twowheel/brighthd.txt"
    #data = np.loadtxt(filename, delimiter=";", usecols=(0,1,2, 3), converters=conv)
    #p.plot(data[:,0], data[:,1], 'k,')

    p.plotGrid()
    plotEcliptic(p)








##############################################################3

##############################################################3

##############################################################3

##############################################################3













def plotEcliptic(maptype=Projection()):
    """Plot Ra and Dec of ecliptic

    Taken from http://www.dur.ac.uk/john.lucey/users/solar_year.html
    His lambda is equal to ra, according to
    http://www.princeton.edu/~achaney/tmve/wiki100k/docs/Ecliptic_coordinate_system.html
    """


    ra = np.empty(360)
    dec = np.empty(360)
    for i in np.arange(360):
        ra[i] = i + 2.45*np.sin (2 * i * np.pi/180.)
        dec[i] =23.5*np.sin( i*np.pi/180.)

    maptype.plotLine(ra, dec, 'r-', lw=4, label="Ecliptic")
